'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _js = require('keccak/js');

var _js2 = _interopRequireDefault(_js);

var _util = require('../util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class EthereumAddress {
    constructor(publicKey) {
        this._publicKey = (0, _util.uncompressPublicKey)(publicKey);
    }
    static from(publicKey) {
        return new EthereumAddress(publicKey);
    }
    static checksumAddress(address) {
        const addrLowerCase = address.toLowerCase();
        if (!EthereumAddress.isValid(addrLowerCase)) {
            throw new Error('invalid address');
        }
        const addr = addrLowerCase.startsWith('0x') ? addrLowerCase.slice(2) : addrLowerCase;
        const hash = (0, _js2.default)('keccak256').update(addr, 'ascii').digest('hex');
        let newAddr = '0x';
        for (let i = 0; i < addr.length; i++) {
            if (hash[i] >= '8') {
                newAddr += addr[i].toUpperCase();
            } else {
                newAddr += addr[i];
            }
        }
        return newAddr;
    }
    static isValid(address) {
        const addr = address.match(/^0[xX]/) ? address.slice(2) : address;
        if (addr.length !== 40) {
            return false;
        }
        if (addr.match(/[0-9a-f]{40}/) || addr.match(/[0-9A-F]{40}/)) {
            return true;
        }
        let checksumAddress;
        try {
            checksumAddress = EthereumAddress.checksumAddress(addr);
        } catch (_err) {
            return false;
        }
        return addr === checksumAddress.slice(2);
    }
    get publicKey() {
        return this._publicKey;
    }
    get rawAddress() {
        if (!this._rawAddress) {
            this._rawAddress = (0, _js2.default)('keccak256').update(this._publicKey.slice(1)).digest().slice(-20);
        }
        return this._rawAddress;
    }
    get address() {
        if (!this._address) {
            const rawAddress = this.rawAddress.toString('hex');
            this._address = EthereumAddress.checksumAddress(rawAddress);
        }
        return this._address;
    }
}
exports.default = EthereumAddress;